module BinderSDK
  class BinderSDKError < StandardError
  end
  class BinderSDKConfigurationError < BinderSDKError
  end

  class Configuration

    SETTINGS = [ 
      :logger
    ]

    attr_writer *SETTINGS

    def initialize
      self.logger = Filum.logger
    end

    [:access_id, :secret_key, :endpoint, :cache].each do |setting|
      define_method "#{setting}=" do |val|
        Loquor.config.send("#{setting}=", val)
      end
    end

    def logger=(val)
      @logger = val
      Loquor.config.logger = val
    end

    SETTINGS.each do |setting|
      define_method setting do
        get_or_raise(setting)
      end
    end

    private

    def get_or_raise(setting)
      instance_variable_get("@#{setting.to_s}") ||
        raise(BinderSDKConfigurationError.new("Configuration for #{setting} is not set"))
    end
  end
end
